@@ -42,7 +42,7 @@
 
 #include "libedataserver/e-dbhash.h"
 #include "libedataserver/e-db3-utils.h"
-#include "libedataserver/e-util.h"
+#include "libedataserver/e-data-server-util.h"
 
 #include "libebook/e-contact.h"
 
@@ -563,13 +563,8 @@ book_view_thread (gpointer data)
 			db_error = db->get (db, NULL, &id_dbt, &vcard_dbt, 0);
 
 			if (db_error == 0) {
-				EContact *contact = create_contact (id_dbt.data, vcard_dbt.data);
 				/* notify_update will check if it matches for us */
-				e_data_book_view_notify_update (book_view, contact);
-				
-				g_object_unref (contact);
-
-				free (vcard_dbt.data);
+				e_data_book_view_notify_update_vcard (book_view, vcard_dbt.data);
 			}
 			else {
 				g_warning ("db->get returned %d", db_error);
@@ -584,6 +579,7 @@ book_view_thread (gpointer data)
 
 		memset (&id_dbt, 0, sizeof (id_dbt));
 		memset (&vcard_dbt, 0, sizeof (vcard_dbt));
+		vcard_dbt.flags = DB_DBT_MALLOC;
 
 		db_error = db->cursor (db, NULL, &dbc, 0);
 		if (db_error == 0) {
@@ -600,10 +596,8 @@ book_view_thread (gpointer data)
 
 				/* don't include the version in the list of cards */
 				if (strcmp (id_dbt.data, E_BOOK_BACKEND_FILE_VERSION_NAME)) {
-					EContact *contact = create_contact (id_dbt.data, vcard_dbt.data);
 					/* notify_update will check if it matches for us */
-					e_data_book_view_notify_update (book_view, contact);
-					g_object_unref (contact);
+					e_data_book_view_notify_update_vcard (book_view, vcard_dbt.data);
 				}
 
 				db_error = dbc->c_get(dbc, &id_dbt, &vcard_dbt, DB_NEXT);
@@ -1377,6 +1371,29 @@ my_open (const char *name, int oflag, ...)
 
 	return g_open (name, oflag, mode);
 }
+
+int
+my_rename (const char *oldname, const char *newname)
+{
+	return g_rename (oldname, newname);
+}
+
+int
+my_exists (const char *name, int *isdirp)
+{
+	if (!g_file_test (name, G_FILE_TEST_EXISTS))
+		return ENOENT;
+	if (isdirp != NULL)
+		*isdirp = g_file_test (name, G_FILE_TEST_IS_DIR);
+	return 0;
+}
+
+int
+my_unlink (const char *name)
+{
+	return g_unlink (name);
+}
+
 #endif
 
 static void
@@ -1413,9 +1430,14 @@ e_book_backend_file_class_init (EBookBackendFileClass *klass)
 	object_class->dispose = e_book_backend_file_dispose;
 
 #ifdef G_OS_WIN32
-	/* Use the gstdio wrapper for open() to open files in libdb */
+	/* Use the gstdio wrappers to open, check, rename and unlink
+	 * files from libdb.
+	 */
 	db_env_set_func_open (my_open);
 	db_env_set_func_close (close);
+	db_env_set_func_exists (my_exists);
+	db_env_set_func_rename (my_rename);
+	db_env_set_func_unlink (my_unlink);
 #endif
 }
 
