@@ -2032,7 +2032,8 @@ e_book_response_get_book_view (EBook       *book,
 
 	op->status = status;
 	op->view = e_book_view_new (corba_book_view, op->listener);
-
+	e_book_view_set_book (op->view, book);
+	
 	bonobo_object_unref(BONOBO_OBJECT(op->listener));
 
 	if (op->synchronous) {
@@ -3192,7 +3193,7 @@ activate_factories_for_uri (EBook *book, const char *uri)
 	char *query;
 	GList *factories = NULL;
 
-	query = "repo_ids.has ('IDL:GNOME/Evolution/DataServer/BookFactory:" BASE_VERSION "')";
+	query = "repo_ids.has ('IDL:GNOME/Evolution/DataServer/BookFactory:" API_VERSION "')";
 
 	CORBA_exception_init (&ev);
 	
@@ -3869,7 +3870,7 @@ e_book_new_system_addressbook    (GError **error)
 		filename = g_build_filename (g_get_home_dir(),
 					     ".evolution/addressbook/local/system",
 					     NULL);
-		uri = g_strdup_printf ("file://%s", filename);
+		uri = g_filename_to_uri (filename, NULL, NULL);
 
 		g_free (filename);
 	
