@@ -7,30 +7,36 @@
  *  Copyright 2003, Novell, Inc.
  *
  * This program is free software; you can redistribute it and/or 
- * modify it under the terms of version 2 of the GNU General Public 
+ * modify it under the terms of version 2 of the GNU Lesser General Public 
  * License as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Lesser General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  * USA
  */
 
-#ifdef HAVE_CONFIG_H
 #include <config.h>
-#endif
 
 #include <string.h>
 #include <sys/types.h>
-#include <glib/gi18n.h>
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <unistd.h>
+
+#ifndef O_BINARY
+#define O_BINARY 0
+#endif
+
+#include <glib.h>
+#include <glib/gi18n.h>
+#include <glib/gstdio.h>
+
 #include <libgnomevfs/gnome-vfs-mime-utils.h>
 #include <e-gw-connection.h>
 #include <e-gw-message.h>
@@ -266,47 +272,16 @@ e_cal_backend_groupwise_set_attachments_from_comp (ECalComponent *comp,
 		
 		EGwItemAttachment *attach_item;
 		char *file_contents, *encoded_data;
-		int fd, len;
-		int len_read = 0;
-		char buf[1024];
-		struct stat sb;
+		int file_len;
 		char *attach_filename_full, *filename;
 		const char *uid;
-		
 
-		attach_filename_full = (char *)l->data + 7;
-		attach_item = g_new0 (EGwItemAttachment, 1);
-		/* FIXME the member does not follow the naming convention.
-		 * Should be fixed in e-gw-item*/
-		attach_item->contentType = g_strdup (gnome_vfs_get_mime_type (attach_filename_full));
-
-		/*
-		 * Would gnome_vfs_async be better suited for this ?
-		 */
-		fd = open (attach_filename_full, O_RDONLY); 
-		if (fd == -1) {
-			g_free (attach_item);
-			g_message ("DEBUG: could not open the file descriptor\n");
-		}
-		if (fstat (fd, &sb) == -1) {
-			g_free (attach_item);
-			g_message ("DEBUG: could not fstat the attachment file\n");
+		attach_filename_full = g_filename_from_uri ((char *)l->data, NULL, NULL);
+		if (!g_file_get_contents (attach_filename_full, &file_contents, &file_len, NULL)) {
+			g_message ("DEBUG: could not read %s\n", attach_filename_full);
+			g_free (attach_filename_full);
 			continue;
 		}
-		len = sb.st_size;
-
-		file_contents = g_malloc (len + 1);
-	
-		while (len_read < len) {
-			int c = read (fd, buf, sizeof (buf));
-
-			if (c == -1)
-				break;
-
-			memcpy (&file_contents[len_read], buf, c);
-			len_read += c;
-		}
-		file_contents [len_read] = 0;
 
 		/* Extract the simple file name from the
 		 * attach_filename_full which is of the form
@@ -314,36 +289,58 @@ e_cal_backend_groupwise_set_attachments_from_comp (ECalComponent *comp,
 		 */
 		e_cal_component_get_uid (comp, &uid);
 		filename = g_strrstr (attach_filename_full, uid); 		
-
 		if (filename == NULL) {
-			g_free (attach_item);
-			g_message ("DEBUG:\n This is an invalid attachment file\n");
+			g_message ("DEBUG: This is an invalid attachment file\n");
+			g_free (attach_filename_full);
+			g_free (file_contents);
 			continue;
-		}	
+		}
 
+		attach_item = g_new0 (EGwItemAttachment, 1);
+		/* FIXME the member does not follow the naming convention.
+		 * Should be fixed in e-gw-item*/
+		attach_item->contentType = g_strdup (gnome_vfs_get_mime_type (attach_filename_full));
+		g_free (attach_filename_full);
 
 		attach_item->name = g_strdup (filename + strlen(uid) + 1);
 		/* do a base64 encoding so it can be embedded in a soap
 		 * message */
-		encoded_data = soup_base64_encode (file_contents, len_read);
+		encoded_data = soup_base64_encode (file_contents, file_len);
 		attach_item->data = encoded_data;
 		attach_item->size = strlen (encoded_data); 
 
 		g_free (file_contents);
-		close (fd);
 		attach_list = g_slist_append (attach_list, attach_item);
 	}
 
 	e_gw_item_set_attach_id_list (item, attach_list);
 }
-	
+
+/* Returns the icalproperty for the Attendee associted with email id */
+static icalproperty *
+get_attendee_prop (icalcomponent *icalcomp, const char *attendee)
+{
+	icalproperty *prop;	
+
+	for (prop = icalcomponent_get_first_property (icalcomp, ICAL_ATTENDEE_PROPERTY);
+			prop;
+			prop = icalcomponent_get_next_property (icalcomp, ICAL_ATTENDEE_PROPERTY)) {
+		const char *att = icalproperty_get_attendee (prop);
+
+		if (!g_ascii_strcasecmp (att, attendee)) {
+			return prop;
+		}
+	}
+
+	return NULL;
+}
+
 /* get_attendee_list from cal comp and convert into
  * egwitemrecipient and set it on recipient_list*/
 static void
 set_attendees_to_item (EGwItem *item, ECalComponent *comp, icaltimezone *default_zone, gboolean delegate, const char *user_email)
 {
 	if (e_cal_component_has_attendees (comp)) {
-		gboolean att_changed = FALSE;
 		GSList *attendee_list, *recipient_list = NULL, *al;
 
 		e_cal_component_get_attendee_list (comp, &attendee_list);	
@@ -355,8 +352,10 @@ set_attendees_to_item (EGwItem *item, ECalComponent *comp, icaltimezone *default
 				continue;		
 			
 			if (delegate) {
-				attendee->delfrom = "";
-				att_changed = TRUE;
+				icalproperty *prop = get_attendee_prop (e_cal_component_get_icalcomponent (comp), 
+						attendee->value);
+				if (prop) 
+					icalproperty_remove_parameter_by_kind (prop, ICAL_DELEGATEDFROM_PARAMETER);
 			}
 	
 			recipient = g_new0 (EGwItemRecipient, 1);
@@ -381,10 +380,6 @@ set_attendees_to_item (EGwItem *item, ECalComponent *comp, icaltimezone *default
 			recipient_list = g_slist_append (recipient_list, recipient);
 		}
 
-		/* Reset the ECalComponent with changes if necessary*/
-		if(att_changed)
-			e_cal_component_set_attendee_list (comp, attendee_list);	
-
 		e_cal_component_free_attendee_list(attendee_list);
 	
 		/* recipient_list shouldn't be freed. Look into the function below. */
@@ -776,27 +771,30 @@ set_attachments_to_cal_component (EGwItem *item, ECalComponent *comp, ECalBacken
 		EGwItemAttachment *attach_item;
 		char *attach_data = NULL;
 		struct stat st;
+		char *filename;
 
 		attach_item = (EGwItemAttachment *) l->data;
 		attach_file_url = g_strconcat (e_cal_backend_groupwise_get_local_attachments_store (cbgw), 
 			 "/", uid, "-", attach_item->name, NULL);
 
-		if (stat (attach_file_url + 7, &st) == -1) {
-			if (!get_attach_data_from_server (attach_item, cbgw))
-				return; /* Could not get the attachment from the server */	
-			fd = open (attach_file_url+7, O_RDWR|O_CREAT|O_TRUNC, 0600);
+		filename = g_filename_from_uri (attach_file_url, NULL, NULL);
+		if (g_stat (filename, &st) == -1) {
+			if (!get_attach_data_from_server (attach_item, cbgw)) {
+				g_free (filename);
+				return; /* Could not get the attachment from the server */
+			}
+			fd = g_open (filename, O_RDWR|O_CREAT|O_TRUNC|O_BINARY, 0600);
 			if (fd == -1) { 
 				/* skip gracefully */
 				g_warning ("DEBUG: could not serialize attachments\n");
-			}
-
-			if (write (fd, attach_item->data, attach_item->size) == -1) {
+			} else if (write (fd, attach_item->data, attach_item->size) == -1) {
 				/* skip gracefully */
 				g_warning ("DEBUG: attachment write failed.\n");
 			}
 			g_free (attach_data);
 			close (fd);
 		}
+		g_free (filename);
 
 		comp_attachment_list = g_slist_append (comp_attachment_list, attach_file_url);
 	}
@@ -805,6 +803,71 @@ set_attachments_to_cal_component (EGwItem *item, ECalComponent *comp, ECalBacken
 
 }
 
+static void
+set_default_alarms (ECalComponent *comp)
+{
+
+	GConfClient *gconf = gconf_client_get_default ();
+
+	if (gconf_client_get_bool (gconf, CALENDAR_CONFIG_DEFAULT_REMINDER, NULL)) {
+
+			ECalComponentAlarm *acomp;
+			int interval;
+			char * units;
+			enum {
+				DAYS,
+				HOURS,
+				MINUTES
+			} duration;	
+			ECalComponentAlarmTrigger trigger;
+
+			interval = gconf_client_get_int (gconf, CALENDAR_CONFIG_DEFAULT_REMINDER_INTERVAL, NULL);
+			units = gconf_client_get_string (gconf, CALENDAR_CONFIG_DEFAULT_REMINDER_UNITS, NULL);
+			g_object_unref (gconf);
+
+			if (units == NULL)
+                                duration = MINUTES;
+			else {
+				if (!strcmp (units, "days"))
+					duration = DAYS;
+				else if (!strcmp (units, "hours"))
+					duration = HOURS;
+				else
+					duration = MINUTES;
+				g_free (units);
+			}
+			acomp = e_cal_component_alarm_new ();
+
+			e_cal_component_alarm_set_action (acomp, E_CAL_COMPONENT_ALARM_DISPLAY);
+
+			trigger.type = E_CAL_COMPONENT_ALARM_TRIGGER_RELATIVE_START;
+			memset (&trigger.u.rel_duration, 0, sizeof (trigger.u.rel_duration));
+
+			trigger.u.rel_duration.is_neg = TRUE;
+
+			switch (duration) {
+			case MINUTES:
+				trigger.u.rel_duration.minutes = interval;
+				break;
+			case HOURS:	
+				trigger.u.rel_duration.hours = interval;
+				break;
+			case DAYS:	
+				trigger.u.rel_duration.days = interval;
+				break;
+			default:
+				e_cal_component_alarm_free (acomp);
+				return;
+			}
+
+			e_cal_component_alarm_set_trigger (acomp, trigger);
+			e_cal_component_add_alarm (comp, acomp);
+
+			e_cal_component_alarm_free (acomp);
+			}
+}
+
+
 ECalComponent *
 e_gw_item_to_cal_component (EGwItem *item, ECalBackendGroupwise *cbgw)
 {
@@ -956,10 +1019,11 @@ e_gw_item_to_cal_component (EGwItem *item, ECalBackendGroupwise *cbgw)
 			}
 		} else {
 			dt.value = &itt_utc;
+			dt.tzid = NULL;
+			dt.value->is_date = 1;
 		}	
 
 		e_cal_component_set_dtstart (comp, &dt);
-		g_free (t);
 	}
 	else 
 		return NULL;
@@ -978,7 +1042,6 @@ e_gw_item_to_cal_component (EGwItem *item, ECalBackendGroupwise *cbgw)
 		recur_id->type = E_CAL_COMPONENT_RANGE_SINGLE;
 		recur_id->datetime = dt;
 		e_cal_component_set_recurid (comp, recur_id);
-
 	} else {
 
 		uid = e_gw_item_get_icalid (item);
@@ -989,6 +1052,8 @@ e_gw_item_to_cal_component (EGwItem *item, ECalBackendGroupwise *cbgw)
 			return NULL;
 		}
 	}
+		
+	g_free (t);
 
 	/* classification */
 	description = e_gw_item_get_classification (item);
@@ -1022,8 +1087,14 @@ e_gw_item_to_cal_component (EGwItem *item, ECalBackendGroupwise *cbgw)
 			 * for getting cutype and the status */
 			attendee->cutype = ICAL_CUTYPE_INDIVIDUAL;
 			 
-			if (recipient->status == E_GW_ITEM_STAT_ACCEPTED)
-				attendee->status = ICAL_PARTSTAT_ACCEPTED;
+			if (recipient->status == E_GW_ITEM_STAT_ACCEPTED) {
+				const char *accept_level = e_gw_item_get_accept_level (item);
+
+				if(accept_level && !strcmp (e_gw_item_get_accept_level (item),"Tentative"))
+					attendee->status = ICAL_PARTSTAT_TENTATIVE;
+				else
+					attendee->status = ICAL_PARTSTAT_ACCEPTED;
+			}
 			else if (recipient->status == E_GW_ITEM_STAT_DECLINED)
 				attendee->status = ICAL_PARTSTAT_DECLINED;
 			else
@@ -1084,6 +1155,8 @@ e_gw_item_to_cal_component (EGwItem *item, ECalBackendGroupwise *cbgw)
 				}
 			} else {
 				dt.value = &itt_utc;
+				dt.tzid = NULL;
+				dt.value->is_date = 1;
 			}
 
 			e_cal_component_set_dtend (comp, &dt);
@@ -1103,9 +1176,9 @@ e_gw_item_to_cal_component (EGwItem *item, ECalBackendGroupwise *cbgw)
 			trigger.u.rel_duration = icaldurationtype_from_int (alarm_duration);
 			e_cal_component_alarm_set_trigger (alarm, trigger);
 			e_cal_component_add_alarm (comp, alarm);
-		}
 
-		
+		} else 
+			set_default_alarms (comp);
 
 		break;
 	case E_GW_ITEM_TYPE_TASK :
@@ -1177,60 +1250,41 @@ e_gw_item_to_cal_component (EGwItem *item, ECalBackendGroupwise *cbgw)
 }
 
 EGwConnectionStatus
-e_gw_connection_send_appointment (ECalBackendGroupwise *cbgw, const char *container, ECalComponent *comp, icalproperty_method method, gboolean *remove, ECalComponent **created_comp)
+e_gw_connection_send_appointment (ECalBackendGroupwise *cbgw, const char *container, ECalComponent *comp, icalproperty_method method, gboolean all_instances, ECalComponent **created_comp, icalparameter_partstat *pstatus)
 {
 	EGwConnection *cnc;
 	EGwConnectionStatus status;
 	icalparameter_partstat partstat;
 	char *item_id = NULL;
 	const char *gw_id;
-	gboolean all_instances = FALSE;
-	icalproperty *icalprop;
-	icalcomponent *icalcomp;
 	const char *recurrence_key = NULL;
+	gboolean need_to_get = FALSE;
 
 	cnc = e_cal_backend_groupwise_get_connection (cbgw);
 	g_return_val_if_fail (E_IS_GW_CONNECTION (cnc), E_GW_CONNECTION_STATUS_INVALID_CONNECTION);
 	g_return_val_if_fail (E_IS_CAL_COMPONENT (comp), E_GW_CONNECTION_STATUS_INVALID_OBJECT);
 
 	e_cal_component_commit_sequence (comp);
-	/* When the icalcomponent is obtained through the itip message rather
-	 * than from the SOAP protocol, the container id has to be explicitly 
-	 * added to the xgwrecordid inorder to obtain the item id. */
 
-	/* handle recurrences - All */
-	icalcomp = e_cal_component_get_icalcomponent (comp);
-
-	icalprop = icalcomponent_get_first_property (icalcomp, ICAL_X_PROPERTY);
-	while (icalprop) {
-		const char *x_name;
-
-		x_name = icalproperty_get_x_name (icalprop);
-		if (!strcmp (x_name, "X-GW-RECUR-INSTANCES-MOD-TYPE")) {
-			if (!strcmp (icalproperty_get_x (icalprop), "All"))
-				all_instances = TRUE;
-			if (recurrence_key)
-				break;
-		}
-		if (!strcmp (x_name, "X-GW-RECURRENCE-KEY")) {
-			e_cal_component_get_uid (comp, &recurrence_key);
-		}
-		icalprop = icalcomponent_get_next_property (icalcomp, ICAL_X_PROPERTY);
-	}	
-		
 	gw_id = e_cal_component_get_gw_id (comp);
 
 	switch  (e_cal_component_get_vtype (comp)) {
 
 		case E_CAL_COMPONENT_EVENT: 
-			if (!g_str_has_suffix (gw_id, container))
+			if (!g_str_has_suffix (gw_id, container)) {
 				item_id = g_strconcat (e_cal_component_get_gw_id (comp), GW_EVENT_TYPE_ID, container, NULL);
+				need_to_get = TRUE;
+				
+			}
 			else 
 				item_id = g_strdup (gw_id);
 			break;
 		case E_CAL_COMPONENT_TODO:
-			if (!g_str_has_suffix (gw_id, container))
+			if (!g_str_has_suffix (gw_id, container)) {
 				item_id = g_strconcat (e_cal_component_get_gw_id (comp), GW_TODO_TYPE_ID, container, NULL);
+				need_to_get = TRUE;
+				
+			}
 			else
 				item_id = g_strdup (gw_id);
 			break;
@@ -1238,6 +1292,21 @@ e_gw_connection_send_appointment (ECalBackendGroupwise *cbgw, const char *contai
 			return E_GW_CONNECTION_STATUS_INVALID_OBJECT;
 	}
 
+	if (all_instances)
+		e_cal_component_get_uid (comp, &recurrence_key);
+
+	/*FIXME - remove this once the server returns us the same iCalid in both interfaces */
+
+	if (need_to_get) {
+		EGwItem *item = NULL;
+		 
+		status = e_gw_connection_get_item (cnc, container, item_id, "recipients message recipientStatus attachments default", &item);
+		if (status == E_GW_CONNECTION_STATUS_OK)
+			*created_comp = e_gw_item_to_cal_component (item, cbgw);
+
+		g_object_unref (item);
+	}
+
 	switch (method) {
 	case ICAL_METHOD_REQUEST:
 		/* get attendee here and add the list along. */
@@ -1252,7 +1321,7 @@ e_gw_connection_send_appointment (ECalBackendGroupwise *cbgw, const char *contai
 				tmp = (ECalComponentAttendee *) (l->data);
 				email_id = tmp->value;
 				
-				if (!g_strncasecmp (email_id, "mailto:", 7))
+				if (!g_ascii_strncasecmp (email_id, "mailto:", 7))
 					email_id += 7;
 
 				if (!g_ascii_strcasecmp (email_id, e_gw_connection_get_user_email (cnc))) {
@@ -1275,6 +1344,7 @@ e_gw_connection_send_appointment (ECalBackendGroupwise *cbgw, const char *contai
 			status = E_GW_CONNECTION_STATUS_INVALID_OBJECT;
 			break;
 		}
+		*pstatus = partstat;
 		switch (partstat) {
 		ECalComponentTransparency transp;
 			
@@ -1299,7 +1369,6 @@ e_gw_connection_send_appointment (ECalBackendGroupwise *cbgw, const char *contai
 			else
 				status = e_gw_connection_decline_request (cnc, item_id, NULL, NULL);
 			
-			*remove = TRUE;
 			break;
 		case ICAL_PARTSTAT_TENTATIVE:
 			if (all_instances)
@@ -1319,23 +1388,11 @@ e_gw_connection_send_appointment (ECalBackendGroupwise *cbgw, const char *contai
 
 	case ICAL_METHOD_CANCEL:
 		status = e_gw_connection_retract_request (cnc, item_id, NULL, FALSE, FALSE);
-		*remove = TRUE;
 		break;
 	default:
 		return E_GW_CONNECTION_STATUS_INVALID_OBJECT;
 	}
 	
-	if (status == E_GW_CONNECTION_STATUS_ITEM_ALREADY_ACCEPTED)
-		return status;
-
-	/*FIXME - handling recurrence items */
-	if (!*remove && status == E_GW_CONNECTION_STATUS_OK) {
-		EGwItem *item;
-
-		status = e_gw_connection_get_item (cnc, container, item_id, "recipients message recipientStatus attachments default", &item);
-		*created_comp = e_gw_item_to_cal_component (item, cbgw);
-	}
-
 	return status;
 }
 
