@@ -3,7 +3,7 @@
  FILE: icaltimezone.c
  CREATOR: Damon Chaplin 15 March 2001
 
- $Id: icaltimezone.c,v 1.22 2003/10/21 18:28:27 ettore Exp $
+ $Id: icaltimezone.c,v 1.23.2.1 2006/05/12 07:11:58 ireneh Exp $
  $Locker:  $
 
  (C) COPYRIGHT 2001, Damon Chaplin
@@ -40,8 +40,10 @@
 #include "icaltimezone.h"
 
 #ifdef WIN32
+#include <mbstring.h>
+#include <sys/stat.h>
+#include <windows.h>
 #define snprintf _snprintf
-#define PACKAGE_DATA_DIR "/Projects/libical"
 #endif
 
 /** This is the toplevel directory where the timezone data is installed in. */
@@ -1330,12 +1332,12 @@ icaltimezone_get_builtin_timezone	(const char *location)
     if (!location || !location[0])
 	return NULL;
 
-    if (!strcmp (location, "UTC"))
-	return &utc_timezone;
-
     if (!builtin_timezones)
 	icaltimezone_init_builtin_timezones ();
 
+    if (!strcmp (location, "UTC"))
+	return &utc_timezone;
+
     /* Do a simple binary search. */
     lower = middle = 0;
     upper = builtin_timezones->num_elements;
@@ -1688,7 +1690,97 @@ format_utc_offset			(int		 utc_offset,
 
 static char* get_zone_directory(void)
 {
+#ifndef WIN32
 	return zone_files_directory == NULL ? ZONEINFO_DIRECTORY : zone_files_directory;
+#else
+	wchar_t wbuffer[1000];
+	char buffer[1000], zoneinfodir[1000], dirname[1000];
+	BOOL used_default;
+	static char *cache = NULL;
+	char *dirslash, *zislash;
+	struct stat st;
+
+	if (zone_files_directory)
+	    return zone_files_directory;
+
+	if (cache)
+	    return cache;
+
+	/* Get the filename of the application */
+	if (!GetModuleFileNameW (NULL, wbuffer, sizeof (wbuffer) / sizeof (wbuffer[0])))
+	    return ZONEINFO_DIRECTORY;
+
+	/* Convert to system codepage */
+	if (!WideCharToMultiByte (CP_ACP, 0, wbuffer, -1, buffer, sizeof (buffer),
+				  NULL, &used_default) ||
+	    used_default) {
+	    /* Failed, try 8.3 format */
+	    if (!GetShortPathNameW (wbuffer, wbuffer,
+				    sizeof (wbuffer) / sizeof (wbuffer[0])) ||
+		!WideCharToMultiByte (CP_ACP, 0, wbuffer, -1, buffer, sizeof (buffer),
+				      NULL, &used_default) ||
+		used_default)
+		return ZONEINFO_DIRECTORY;
+	}
+
+	/* Look for the zoneinfo directory somewhere in the path where
+	 * the app is installed. If the path to the app is
+	 *
+	 *	C:\opt\evo-2.6\bin\evolution-2.6.exe 
+	 *
+	 * and the compile-time ZONEINFO_DIRECTORY is
+	 *
+	 *	C:/devel/target/evo/share/evolution-data-server-1.6/zoneinfo,
+	 *
+	 * we check the pathnames:
+	 *
+	 *	C:\opt\evo-2.6/devel/target/evo/share/evolution-data-server-1.6/zoneinfo
+	 *	C:\opt\evo-2.6/target/evo/share/evolution-data-server-1.6/zoneinfo
+	 *	C:\opt\evo-2.6/evo/share/evolution-data-server-1.6/zoneinfo
+	 *	C:\opt\evo-2.6/share/evolution-data-server-1.6/zoneinfo		<===
+	 *	C:\opt\evo-2.6/evolution-data-server-1.6/zoneinfo
+	 *	C:\opt\evo-2.6/zoneinfo
+	 *	C:\opt/devel/target/evo/share/evolution-data-server-1.6/zoneinfo
+	 *	C:\opt/target/evo/share/evolution-data-server-1.6/zoneinfo
+	 *	C:\opt/evo/share/evolution-data-server-1.6/zoneinfo
+	 *	C:\opt/share/evolution-data-server-1.6/zoneinfo
+	 *	C:\opt/evolution-data-server-1.6/zoneinfo
+	 *	C:\opt/zoneinfo
+	 *	C:/devel/target/evo/share/evolution-data-server-1.6/zoneinfo
+	 *	C:/target/evo/share/evolution-data-server-1.6/zoneinfo
+	 *	C:/evo/share/evolution-data-server-1.6/zoneinfo
+	 *	C:/share/evolution-data-server-1.6/zoneinfo
+	 *	C:/evolution-data-server-1.6/zoneinfo
+	 *	C:/zoneinfo
+	 *
+	 * In Evolution's case, we would get a match already at the
+	 * fourth pathname check.
+	 */
+
+	/* Strip away basename of app .exe first */
+	dirslash = _mbsrchr (buffer, '\\');
+	if (dirslash)
+	    *dirslash = '\0';
+
+	while ((dirslash = _mbsrchr (buffer, '\\'))) {
+	    /* Strip one more directory from app .exe location */
+	    *dirslash = '\0';
+	    
+	    strcpy (zoneinfodir, ZONEINFO_DIRECTORY);
+	    while ((zislash = _mbschr (zoneinfodir, '/'))) {
+		*zislash = '.';
+		strcpy (dirname, buffer);
+		strcat (dirname, "/");
+		strcat (dirname, zislash + 1);
+		if (stat (dirname, &st) == 0 &&
+		    S_ISDIR (st.st_mode)) {
+		    cache = strdup (dirname);
+		    return cache;
+		}
+	    }
+	}
+	return ZONEINFO_DIRECTORY;
+#endif
 }
 
 void set_zone_directory(char *path)
