@@ -30,8 +30,9 @@
 #include <ctype.h>
 
 #include "libedataserver/e-trie.h"
-#include "camel-utf8.h"
+
 #include "camel-url-scanner.h"
+#include "camel-utf8.h"
 
 
 struct _CamelUrlScanner {
@@ -196,7 +197,8 @@ camel_url_addrspec_start (const char *in, const char *pos, const char *inend, ur
 	
 	g_assert (*inptr == '@');
 	
-	inptr--;
+	if (inptr > in)
+		inptr--;
 	
 	while (inptr > in) {
 		if (is_atom (*inptr))
@@ -211,10 +213,10 @@ camel_url_addrspec_start (const char *in, const char *pos, const char *inend, ur
 			inptr--;
 	}
 	
-	if (!is_atom (*inptr) || is_open_brace (*inptr))
+	while (!is_atom (*inptr) || is_open_brace (*inptr))
 		inptr++;
 	
-	if (inptr == pos)
+	if (inptr >= pos)
 		return FALSE;
 	
 	match->um_so = (inptr - in);
@@ -355,8 +357,8 @@ camel_url_web_end (const char *in, const char *pos, const char *inend, urlmatch_
 			while (inptr < inend && is_atom (*inptr))
 				inptr++;
 			
-			if ((inptr + 1) < inend && *inptr == '.' && is_atom (inptr[1]))
-				inptr++;
+			if ((inptr + 1) < inend && *inptr == '.' && (is_atom (inptr[1]) || inptr[1] == '/')) 
+					inptr++;
 		}
 		
 		if (*inptr != '@')
@@ -376,8 +378,8 @@ camel_url_web_end (const char *in, const char *pos, const char *inend, urlmatch_
 			while (inptr < inend && is_domain (*inptr))
 				inptr++;
 			
-			if ((inptr + 1) < inend && *inptr == '.' && is_domain (inptr[1]))
-				inptr++;
+			if ((inptr + 1) < inend && *inptr == '.' && (is_domain (inptr[1]) || inptr[1] == '/')) 
+					inptr++;
 		}
 	} else {
 		return FALSE;
