@@ -9,7 +9,7 @@
  *  Copyright 2000, 2003 Ximian, Inc.
  *
  * This program is free software; you can redistribute it and/or
- * modify it under the terms of version 2 of the GNU General Public
+ * modify it under the terms of version 2 of the GNU Lesser General Public
  * License as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -17,16 +17,14 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
- * You should have received a copy of the GNU General Public
+ * You should have received a copy of the GNU Lesser General Public
  * License along with this program; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
  */
 
-#ifdef HAVE_CONFIG_H
 #include <config.h>
-#endif
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -34,33 +32,40 @@
 #include <unistd.h>
 #include <errno.h>
 
-#include "camel-imap-store.h"
-#include "camel-imap-store-summary.h"
-#include "camel-imap-folder.h"
-#include "camel-imap-utils.h"
+#include <glib.h>
+#include <glib/gstdio.h>
+
+#include "camel/camel-debug.h"
+#include "camel/camel-disco-diary.h"
+#include "camel/camel-exception.h"
+#include "camel/camel-file-utils.h"
+#include "camel/camel-folder.h"
+#include "camel/camel-i18n.h"
+#include "camel/camel-net-utils.h"
+#include "camel/camel-private.h"
+#include "camel/camel-sasl.h"
+#include "camel/camel-session.h"
+#include "camel/camel-stream-buffer.h"
+#include "camel/camel-stream-fs.h"
+#include "camel/camel-stream-process.h"
+#include "camel/camel-stream.h"
+#include "camel/camel-string-utils.h"
+#include "camel/camel-tcp-stream-raw.h"
+#include "camel/camel-tcp-stream-ssl.h"
+#include "camel/camel-url.h"
+#include "camel/camel-utf8.h"
+
 #include "camel-imap-command.h"
-#include "camel-imap-summary.h"
+#include "camel-imap-folder.h"
 #include "camel-imap-message-cache.h"
-#include "camel-disco-diary.h"
-#include "camel-file-utils.h"
-#include "camel-folder.h"
-#include "camel-exception.h"
-#include "camel-session.h"
-#include "camel-stream.h"
-#include "camel-stream-buffer.h"
-#include "camel-stream-fs.h"
-#include "camel-stream-process.h"
-#include "camel-tcp-stream-raw.h"
-#include "camel-tcp-stream-ssl.h"
-#include "camel-url.h"
-#include "camel-sasl.h"
-#include "camel-utf8.h"
-#include "camel-string-utils.h"
-#include "camel-imap-private.h"
-#include "camel-private.h"
-#include "camel-debug.h"
-#include "camel-i18n.h"
-#include "camel-net-utils.h"
+#include "camel-imap-store-summary.h"
+#include "camel-imap-store.h"
+#include "camel-imap-summary.h"
+#include "camel-imap-utils.h"
+
+#if !GLIB_CHECK_VERSION (2, 8, 0)
+#define g_access access
+#endif
 
 #define d(x) 
 
@@ -68,6 +73,14 @@
 #define IMAP_PORT "143"
 #define IMAPS_PORT "993"
 
+#ifdef G_OS_WIN32
+/* The strtok() in Microsoft's C library is MT-safe (but still uses
+ * only one buffer pointer per thread, but for the use of strtok_r()
+ * here that's enough).
+ */
+#define strtok_r(s,sep,lasts) (*(lasts)=strtok((s),(sep)))
+#endif
+
 static CamelDiscoStoreClass *parent_class = NULL;
 
 static char imap_tag_prefix = 'A';
@@ -178,6 +191,7 @@ static void
 camel_imap_store_finalize (CamelObject *object)
 {
 	CamelImapStore *imap_store = CAMEL_IMAP_STORE (object);
+	CamelDiscoStore *disco = CAMEL_DISCO_STORE (object);
 
 	/* This frees current_folder, folders, authtypes, streams, and namespace. */
 	camel_service_disconnect((CamelService *)imap_store, TRUE, NULL);
@@ -191,6 +205,12 @@ camel_imap_store_finalize (CamelObject *object)
 		g_free (imap_store->base_url);
 	if (imap_store->storage_path)
 		g_free (imap_store->storage_path);
+
+	if (disco->diary) {
+		camel_object_unref (disco->diary);
+		disco->diary = NULL;
+	}
+
 }
 
 static void
@@ -239,7 +259,8 @@ construct (CamelService *service, CamelSession *session,
 {
 	CamelImapStore *imap_store = CAMEL_IMAP_STORE (service);
 	CamelStore *store = CAMEL_STORE (service);
-	char *tmp;
+	CamelDiscoStore *disco_store = CAMEL_DISCO_STORE (service);
+	char *tmp, *path;
 	CamelURL *summary_url;
 
 	CAMEL_SERVICE_CLASS (parent_class)->construct (service, session, provider, url, ex);
@@ -258,7 +279,7 @@ construct (CamelService *service, CamelSession *session,
 	imap_store->parameters = 0;
 	if (camel_url_get_param (url, "use_lsub"))
 		imap_store->parameters |= IMAP_PARAM_SUBSCRIPTIONS;
-	if (camel_url_get_param (url, "namespace")) {
+	if (camel_url_get_param (url, "override_namespace") && camel_url_get_param (url, "namespace")) {
 		imap_store->parameters |= IMAP_PARAM_OVERRIDE_NAMESPACE;
 		g_free(imap_store->namespace);
 		imap_store->namespace = g_strdup (camel_url_get_param (url, "namespace"));
@@ -274,6 +295,11 @@ construct (CamelService *service, CamelSession *session,
 	if (camel_url_get_param (url, "filter_junk_inbox"))
 		imap_store->parameters |= IMAP_PARAM_FILTER_JUNK_INBOX;
 
+	/* setup journal*/
+	path = g_strdup_printf ("%s/journal", imap_store->storage_path);
+	disco_store->diary = camel_disco_diary_new (disco_store, path, ex);
+	g_free (path);
+
 	/* setup/load the store summary */
 	tmp = alloca(strlen(imap_store->storage_path)+32);
 	sprintf(tmp, "%s/.ev-store-summary", imap_store->storage_path);
@@ -537,7 +563,7 @@ connect_to_server (CamelService *service, struct addrinfo *ai, int ssl_mode, Cam
 	CamelStream *tcp_stream;
 	CamelSockOptData sockopt;
 	gboolean force_imap4 = FALSE;
-	int clean_quit = TRUE, ret;
+	gboolean clean_quit = TRUE;
 	char *buf;
 	
 	if (ssl_mode != MODE_CLEAR) {
@@ -558,7 +584,7 @@ connect_to_server (CamelService *service, struct addrinfo *ai, int ssl_mode, Cam
 		tcp_stream = camel_tcp_stream_raw_new ();
 	}
 	
-	if ((ret = camel_tcp_stream_connect ((CamelTcpStream *) tcp_stream, ai)) == -1) {
+	if (camel_tcp_stream_connect ((CamelTcpStream *) tcp_stream, ai) == -1) {
 		if (errno == EINTR)
 			camel_exception_set (ex, CAMEL_EXCEPTION_USER_CANCEL,
 					     _("Connection cancelled"));
@@ -746,6 +772,10 @@ connect_to_server (CamelService *service, struct addrinfo *ai, int ssl_mode, Cam
 	return FALSE;
 }
 
+#ifndef G_OS_WIN32
+
+/* Using custom commands to connect to IMAP servers is not supported on Win32 */
+
 static gboolean
 connect_to_server_process (CamelService *service, const char *cmd, CamelException *ex)
 {
@@ -889,6 +919,8 @@ connect_to_server_process (CamelService *service, const char *cmd, CamelExceptio
 	
 }
 
+#endif
+
 static struct {
 	char *value;
 	char *serv;
@@ -905,16 +937,19 @@ static struct {
 static gboolean
 connect_to_server_wrapper (CamelService *service, CamelException *ex)
 {
-	const char *command, *ssl_mode;
+	const char *ssl_mode;
 	struct addrinfo hints, *ai;
 	int mode, ret, i;
 	char *serv;
 	const char *port;
 
+#ifndef G_OS_WIN32
+	const char *command;
+
 	if (camel_url_get_param(service->url, "use_command")
 	    && (command = camel_url_get_param(service->url, "command")))
 		return connect_to_server_process(service, command, ex);
-	
+#endif	
 	if ((ssl_mode = camel_url_get_param (service->url, "use_ssl"))) {
 		for (i = 0; ssl_options[i].value; i++)
 			if (!strcmp (ssl_options[i].value, ssl_mode))
@@ -942,16 +977,11 @@ connect_to_server_wrapper (CamelService *service, CamelException *ex)
 		camel_exception_clear (ex);
 		ai = camel_getaddrinfo(service->url->host, port, &hints, ex);
 	}
+	
 	if (ai == NULL)
 		return FALSE;
 	
-	if (!(ret = connect_to_server (service, ai, mode, ex)) && mode == MODE_SSL) {
-		camel_exception_clear (ex);
-		ret = connect_to_server (service, ai, MODE_TLS, ex);
-	} else if (!ret && mode == MODE_TLS) {
-		camel_exception_clear (ex);
-		ret = connect_to_server (service, ai, MODE_CLEAR, ex);
-	}
+	ret = connect_to_server (service, ai, mode, ex);
 	
 	camel_freeaddrinfo (ai);
 	
@@ -1075,7 +1105,7 @@ imap_forget_folder (CamelImapStore *imap_store, const char *folder_name, CamelEx
 	storage_path = g_strdup_printf ("%s/folders", imap_store->storage_path);
 	folder_dir = imap_path_to_physical (storage_path, folder_name);
 	g_free (storage_path);
-	if (access (folder_dir, F_OK) != 0) {
+	if (g_access (folder_dir, F_OK) != 0) {
 		g_free (folder_dir);
 		goto event;
 	}
@@ -1095,22 +1125,22 @@ imap_forget_folder (CamelImapStore *imap_store, const char *folder_name, CamelEx
 	camel_object_unref (cache);
 	camel_object_unref (summary);
 	
-	unlink (summary_file);
+	g_unlink (summary_file);
 	g_free (summary_file);
 	
 	journal_file = g_strdup_printf ("%s/journal", folder_dir);
-	unlink (journal_file);
+	g_unlink (journal_file);
 	g_free (journal_file);
 
 	state_file = g_strdup_printf ("%s/cmeta", folder_dir);
-	unlink (state_file);
+	g_unlink (state_file);
 	g_free (state_file);
 
 	state_file = g_strdup_printf("%s/subfolders", folder_dir);
-	rmdir(state_file);
+	g_rmdir(state_file);
 	g_free(state_file);
 	
-	rmdir (folder_dir);
+	g_rmdir (folder_dir);
 	g_free (folder_dir);
 	
  event:
@@ -1145,6 +1175,7 @@ imap_check_folder_still_extant (CamelImapStore *imap_store, const char *full_nam
 	return TRUE;
 }
 
+#if 0
 /* This is a little 'hack' to avoid the deadlock conditions that would otherwise
    ensue when calling camel_folder_refresh_info from inside a lock */
 /* NB: on second thougts this is probably not entirely safe, but it'll do for now */
@@ -1191,7 +1222,8 @@ imap_store_refresh_folders (CamelImapStore *store, CamelException *ex)
 	}
 	
 	g_ptr_array_free (folders, TRUE);
-}	
+}
+#endif
 
 static gboolean
 try_auth (CamelImapStore *store, const char *mech, CamelException *ex)
@@ -1215,7 +1247,7 @@ try_auth (CamelImapStore *store, const char *mech, CamelException *ex)
 		
 		sasl_resp = camel_sasl_challenge_base64 (sasl, imap_next_word (resp), ex);
 		g_free (resp);
-		if (camel_exception_is_set (ex))
+		if (!sasl_resp || camel_exception_is_set (ex))
 			goto break_and_lose;
 		
 		response = camel_imap_command_continuation (store, sasl_resp, strlen (sasl_resp), ex);
@@ -1376,10 +1408,9 @@ static gboolean
 imap_connect_online (CamelService *service, CamelException *ex)
 {
 	CamelImapStore *store = CAMEL_IMAP_STORE (service);
-	CamelDiscoStore *disco_store = CAMEL_DISCO_STORE (service);
 	CamelImapResponse *response;
 	/*struct _namespaces *namespaces;*/
-	char *result, *name, *path;
+	char *result, *name;
 	size_t len;
 	CamelImapStoreNamespace *ns;
 
@@ -1435,7 +1466,7 @@ imap_connect_online (CamelService *service, CamelException *ex)
 			 * for the given path, even if that path doesn't exist.
 			 */
 			response = camel_imap_command (store, NULL, ex,
-						       "LIST %S \"\"",
+						       "LIST %G \"\"",
 						       store->namespace);
 		} else {
 			/* Plain IMAP4 doesn't have that idiom, so we fall back
@@ -1443,7 +1474,7 @@ imap_connect_online (CamelService *service, CamelException *ex)
 			 * the folder doesn't exist (eg, if namespace is "").
 			 */
 			response = camel_imap_command (store, NULL, ex,
-						       "LIST \"\" %S",
+						       "LIST \"\" %G",
 						       store->namespace);
 		}
 		if (!response)
@@ -1502,9 +1533,6 @@ imap_connect_online (CamelService *service, CamelException *ex)
 		store->refresh_stamp = time(0);
 	}
 	
-	path = g_strdup_printf ("%s/journal", store->storage_path);
-	disco_store->diary = camel_disco_diary_new (disco_store, path, ex);
-	g_free (path);
 	
  done:
 	/* save any changes we had */
@@ -1523,11 +1551,7 @@ imap_connect_offline (CamelService *service, CamelException *ex)
 {
 	CamelImapStore *store = CAMEL_IMAP_STORE (service);
 	CamelDiscoStore *disco_store = CAMEL_DISCO_STORE (service);
-	char *path;
 
-	path = g_strdup_printf ("%s/journal", store->storage_path);
-	disco_store->diary = camel_disco_diary_new (disco_store, path, ex);
-	g_free (path);
 	if (!disco_store->diary)
 		return FALSE;
 	
@@ -1539,7 +1563,6 @@ static gboolean
 imap_disconnect_offline (CamelService *service, gboolean clean, CamelException *ex)
 {
 	CamelImapStore *store = CAMEL_IMAP_STORE (service);
-	CamelDiscoStore *disco = CAMEL_DISCO_STORE (service);
 
 	if (store->istream) {
 		camel_stream_close(store->istream);
@@ -1570,12 +1593,7 @@ imap_disconnect_offline (CamelService *service, gboolean clean, CamelException *
 		g_free (store->namespace);
 		store->namespace = NULL;
 	}
-	
-	if (disco->diary) {
-		camel_object_unref (disco->diary);
-		disco->diary = NULL;
-	}
-	
+		
 	return TRUE;
 }
 
@@ -1619,19 +1637,13 @@ static void
 imap_noop (CamelStore *store, CamelException *ex)
 {
 	CamelImapStore *imap_store = (CamelImapStore *) store;
-	CamelDiscoStore *disco = (CamelDiscoStore *) store;
 	CamelImapResponse *response;
 	CamelFolder *current_folder;
 	
-	if (camel_disco_store_status (disco) != CAMEL_DISCO_STORE_ONLINE)
-		return;
-	
 	CAMEL_SERVICE_LOCK (imap_store, connect_lock);
 
-	if (!camel_imap_store_connected(imap_store, ex)) {
-		CAMEL_SERVICE_UNLOCK(imap_store, connect_lock);
-		return;
-	}
+	if (!camel_imap_store_connected(imap_store, ex))
+		goto done;
 
 	current_folder = imap_store->current_folder;
 	if (current_folder && imap_summary_is_dirty (current_folder->summary)) {
@@ -1642,7 +1654,7 @@ imap_noop (CamelStore *store, CamelException *ex)
 		if (response)
 			camel_imap_response_free (imap_store, response);
 	}
-	
+done:
 	CAMEL_SERVICE_UNLOCK (imap_store, connect_lock);
 }
 
@@ -1884,7 +1896,7 @@ get_folder_online (CamelStore *store, const char *folder_name, guint32 flags, Ca
 			guint32 flags;
 			int i;
 			
-			if (!(response = camel_imap_command (imap_store, NULL, ex, "LIST \"\" %S", parent_real))) {
+			if (!(response = camel_imap_command (imap_store, NULL, ex, "LIST \"\" %G", parent_real))) {
 				CAMEL_SERVICE_UNLOCK (imap_store, connect_lock);
 				g_free (parent_name);
 				g_free (parent_real);
@@ -1949,7 +1961,7 @@ get_folder_online (CamelStore *store, const char *folder_name, guint32 flags, Ca
 				
 				/* add the dirsep to the end of parent_name */
 				name = g_strdup_printf ("%s%c", parent_real, imap_store->dir_sep);
-				response = camel_imap_command (imap_store, NULL, ex, "CREATE %S",
+				response = camel_imap_command (imap_store, NULL, ex, "CREATE %G",
 							       name);
 				g_free (name);
 				
@@ -1968,7 +1980,7 @@ get_folder_online (CamelStore *store, const char *folder_name, guint32 flags, Ca
 		g_free (parent_name);
 
 		folder_real = camel_imap_store_summary_path_to_full(imap_store->summary, folder_name, imap_store->dir_sep);
-		response = camel_imap_command (imap_store, NULL, ex, "CREATE %S", folder_real);
+		response = camel_imap_command (imap_store, NULL, ex, "CREATE %G", folder_real);
 		if (response) {
 			camel_imap_store_summary_add_from_full(imap_store->summary, folder_real, imap_store->dir_sep);
 
@@ -2134,7 +2146,7 @@ rename_folder_info (CamelImapStore *imap_store, const char *old_name, const char
 			if (imap_store->dir_sep == '.') {
 				CamelImapResponse *response;
 
-				response = camel_imap_command (imap_store, NULL, NULL, "RENAME %F %S", path, nfull);
+				response = camel_imap_command (imap_store, NULL, NULL, "RENAME %F %G", path, nfull);
 				if (response)
 					camel_imap_response_free (imap_store, response);
 			}
@@ -2200,7 +2212,7 @@ rename_folder (CamelStore *store, const char *old_name, const char *new_name_in,
 	g_free(storage_path);
 
 	/* So do we care if this didn't work?  Its just a cache? */
-	if (rename (oldpath, newpath) == -1) {
+	if (g_rename (oldpath, newpath) == -1) {
 		g_warning ("Could not rename message cache '%s' to '%s': %s: cache reset",
 			   oldpath, newpath, strerror (errno));
 	}
@@ -2252,7 +2264,7 @@ create_folder (CamelStore *store, const char *parent_name,
 	}
 
 	need_convert = FALSE;
-	response = camel_imap_command (imap_store, NULL, ex, "LIST \"\" %S",
+	response = camel_imap_command (imap_store, NULL, ex, "LIST \"\" %G",
 				       parent_real);
 	if (!response) /* whoa, this is bad */ {
 		g_free(parent_real);
@@ -2312,7 +2324,7 @@ create_folder (CamelStore *store, const char *parent_name,
 		
 		/* add the dirsep to the end of parent_name */
 		name = g_strdup_printf ("%s%c", parent_real, imap_store->dir_sep);
-		response = camel_imap_command (imap_store, NULL, ex, "CREATE %S",
+		response = camel_imap_command (imap_store, NULL, ex, "CREATE %G",
 					       name);
 		g_free (name);
 		
@@ -2329,7 +2341,7 @@ create_folder (CamelStore *store, const char *parent_name,
 	real_name = camel_imap_store_summary_path_to_full(imap_store->summary, folder_name, imap_store->dir_sep);
 	full_name = imap_concat (imap_store, parent_real, real_name);
 	g_free(real_name);
-	response = camel_imap_command (imap_store, NULL, ex, "CREATE %S", full_name);
+	response = camel_imap_command (imap_store, NULL, ex, "CREATE %G", full_name);
 	
 	if (response) {
 		CamelImapStoreInfo *si;
@@ -2492,7 +2504,7 @@ get_folders_sync(CamelImapStore *imap_store, const char *pattern, CamelException
 	present = g_hash_table_new(folder_hash, folder_eq);
 	for (j=0;j<2;j++) {
 		response = camel_imap_command (imap_store, NULL, ex,
-					       "%s \"\" %S", j==1 ? "LSUB" : "LIST",
+					       "%s \"\" %G", j==1 ? "LSUB" : "LIST",
 					       pattern);
 		if (!response)
 			goto fail;
@@ -2605,7 +2617,10 @@ refresh_refresh(CamelSession *session, CamelSessionThreadMsg *msg)
 
 	if (store->namespace && store->namespace[0]) {
 		char *pattern;
-
+		
+		get_folders_sync(store, "INBOX", &m->ex);
+		if (camel_exception_is_set(&m->ex))
+			goto done;
 		get_folders_sync(store, store->namespace, &m->ex);
 		if (camel_exception_is_set(&m->ex))
 			goto done;
@@ -2655,7 +2670,7 @@ get_folder_info_online (CamelStore *store, const char *top, guint32 flags, Camel
 		time_t now;
 		int ref;
 
-		now = time(0);
+		now = time(NULL);
 		ref = now > imap_store->refresh_stamp+60*60*1;
 		if (ref) {
 			CAMEL_SERVICE_LOCK(store, connect_lock);
@@ -2684,6 +2699,10 @@ get_folder_info_online (CamelStore *store, const char *top, guint32 flags, Camel
 
 		if (top[0] == 0) {
 			if (imap_store->namespace && imap_store->namespace[0]) {
+				get_folders_sync(imap_store, "INBOX", ex);
+				if (camel_exception_is_set(ex))
+					goto fail;
+				
 				i = strlen(imap_store->namespace)-1;
 				pattern = g_alloca(i+5);
 				strcpy(pattern, imap_store->namespace);
@@ -2698,7 +2717,7 @@ get_folder_info_online (CamelStore *store, const char *top, guint32 flags, Camel
 			}
 		} else {
 			char *name;
-
+			
 			name = camel_imap_store_summary_full_from_path(imap_store->summary, top);
 			if (name == NULL)
 				name = camel_imap_store_summary_path_to_full(imap_store->summary, top, imap_store->dir_sep);
@@ -2735,6 +2754,7 @@ get_folder_info_offline (CamelStore *store, const char *top,
 			 guint32 flags, CamelException *ex)
 {
 	CamelImapStore *imap_store = CAMEL_IMAP_STORE (store);
+	gboolean include_inbox = FALSE;
 	CamelFolderInfo *fi;
 	GPtrArray *folders;
 	char *pattern, *name;
@@ -2747,12 +2767,14 @@ get_folder_info_offline (CamelStore *store, const char *top,
 
 	folders = g_ptr_array_new ();
 
-	if (top == NULL)
+	if (top == NULL || top[0] == '\0') {
+		include_inbox = TRUE;
 		top = "";
+	}
 
 	/* get starting point */
 	if (top[0] == 0) {
-		if (imap_store->namespace) {
+		if (imap_store->namespace && imap_store->namespace[0]) {
 			name = g_strdup(imap_store->summary->namespace->full_name);
 			top = imap_store->summary->namespace->path;
 		} else
@@ -2776,9 +2798,10 @@ get_folder_info_offline (CamelStore *store, const char *top,
 
 		if (si == NULL)
 			continue;
-
+		
 		if ((!strcmp(name, camel_imap_store_info_full_name(imap_store->summary, si))
-		     || imap_match_pattern(imap_store->dir_sep, pattern, camel_imap_store_info_full_name(imap_store->summary, si)))
+		     || imap_match_pattern(imap_store->dir_sep, pattern, camel_imap_store_info_full_name(imap_store->summary, si))
+		     || (include_inbox && !g_ascii_strcasecmp (camel_imap_store_info_full_name(imap_store->summary, si), "INBOX")))
 		    && ((imap_store->parameters & IMAP_PARAM_SUBSCRIPTIONS) == 0
 			|| (flags & CAMEL_STORE_FOLDER_INFO_SUBSCRIBED) == 0
 			|| (si->flags & CAMEL_STORE_INFO_FOLDER_SUBSCRIBED))) {
@@ -2791,11 +2814,11 @@ get_folder_info_offline (CamelStore *store, const char *top,
 			   it.  See create folder */
 			if (fi->flags & CAMEL_FOLDER_NOINFERIORS)
 				fi->flags = (fi->flags & ~CAMEL_FOLDER_NOINFERIORS) | CAMEL_FOLDER_NOCHILDREN;
-
+			
 			/* blah, this gets lost somewhere, i can't be bothered finding out why */
 			if (!g_ascii_strcasecmp(fi->full_name, "inbox"))
 				fi->flags = (fi->flags & ~CAMEL_FOLDER_TYPE_MASK) | CAMEL_FOLDER_TYPE_INBOX;
-
+			
 			if (si->flags & CAMEL_FOLDER_NOSELECT) {
 				CamelURL *url = camel_url_new(fi->uri, NULL);
 				
@@ -2845,15 +2868,15 @@ subscribe_folder (CamelStore *store, const char *folder_name,
 	CamelFolderInfo *fi;
 	CamelStoreInfo *si;
 
-	if (!camel_disco_store_check_online (CAMEL_DISCO_STORE (store), ex))
-		return;
+	CAMEL_SERVICE_LOCK(store, connect_lock);
+
 	if (!camel_imap_store_connected (imap_store, ex))
-		return;
+		goto done;
 	
 	response = camel_imap_command (imap_store, NULL, ex,
 				       "SUBSCRIBE %F", folder_name);
 	if (!response)
-		return;
+		goto done;
 	camel_imap_response_free (imap_store, response);
 	
 	si = camel_store_summary_path((CamelStoreSummary *)imap_store->summary, folder_name);
@@ -2870,7 +2893,7 @@ subscribe_folder (CamelStore *store, const char *folder_name,
 		/* we don't need to emit a "folder_subscribed" signal
                    if we are in the process of renaming folders, so we
                    are done here... */
-		return;
+		goto done;
 	}
 
 	fi = imap_build_folder_info(imap_store, folder_name);
@@ -2878,6 +2901,8 @@ subscribe_folder (CamelStore *store, const char *folder_name,
 	
 	camel_object_trigger_event (CAMEL_OBJECT (store), "folder_subscribed", fi);
 	camel_folder_info_free (fi);
+done:
+	CAMEL_SERVICE_UNLOCK(store, connect_lock);
 }
 
 static void
@@ -2886,19 +2911,21 @@ unsubscribe_folder (CamelStore *store, const char *folder_name,
 {
 	CamelImapStore *imap_store = CAMEL_IMAP_STORE (store);
 	CamelImapResponse *response;
+
+	CAMEL_SERVICE_LOCK(store, connect_lock);
 	
-	if (!camel_disco_store_check_online (CAMEL_DISCO_STORE (store), ex))
-		return;
 	if (!camel_imap_store_connected (imap_store, ex))
-		return;
+		goto done;
 	
 	response = camel_imap_command (imap_store, NULL, ex,
 				       "UNSUBSCRIBE %F", folder_name);
 	if (!response)
-		return;
+		goto done;
 	camel_imap_response_free (imap_store, response);
 
 	imap_folder_effectively_unsubscribed (imap_store, folder_name, ex);
+done:
+	CAMEL_SERVICE_UNLOCK(store, connect_lock);
 }
 
 #if 0
@@ -2953,7 +2980,7 @@ ssize_t
 camel_imap_store_readline (CamelImapStore *store, char **dest, CamelException *ex)
 {
 	CamelStreamBuffer *stream;
-	char linebuf[1024];
+	char linebuf[1024] = {0};
 	GByteArray *ba;
 	ssize_t nread;
 	
