@@ -5,7 +5,7 @@
  *  Copyright 2000 Ximian, Inc. (www.ximian.com)
  *
  * This program is free software; you can redistribute it and/or
- * modify it under the terms of version 2 of the GNU General Public
+ * modify it under the terms of version 2 of the GNU Lesser General Public
  * License as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -13,7 +13,7 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
- * You should have received a copy of the GNU General Public
+ * You should have received a copy of the GNU Lesser General Public
  * License along with this program; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
@@ -31,7 +31,6 @@
 #include <time.h>
 #include <errno.h>
 #include <sys/types.h>
-#include <dirent.h>
 #include <sys/stat.h>
 #include <unistd.h>
 
@@ -42,6 +41,8 @@
 #include "camel-string-utils.h"
 #include "camel-utf8.h"
 
+#include <glib/gstdio.h>
+
 #define d(x)
 
 #define SUBFOLDER_DIR_NAME     "subfolders"
@@ -380,7 +381,7 @@ imap_parse_list_response (CamelImapStore *store, const char *buf, int *flags, ch
 			return FALSE;
 
 		*folder = astring;
-#if 0
+
 		char *mailbox;
 
 		mailbox = imap_mailbox_decode (astring, strlen (astring));
@@ -404,7 +405,7 @@ imap_parse_list_response (CamelImapStore *store, const char *buf, int *flags, ch
 			*flags &= ~CAMEL_FOLDER_NOSELECT;
 		
 		*folder = mailbox;
-#endif
+
 	}
 	
 	return TRUE;
@@ -478,6 +479,18 @@ imap_create_flag_list (guint32 flags)
 		g_string_append (gstr, "\\Flagged ");
 	if (flags & CAMEL_MESSAGE_SEEN)
 		g_string_append (gstr, "\\Seen ");
+	if (flags & CAMEL_MESSAGE_JUNK)
+		g_string_append (gstr, "Junk ");
+	if (flags & CAMEL_IMAP_MESSAGE_LABEL1)
+		g_string_append(gstr, "$Label1 ");
+	if (flags & CAMEL_IMAP_MESSAGE_LABEL2)
+		g_string_append(gstr, "$Label2 ");
+	if (flags & CAMEL_IMAP_MESSAGE_LABEL3)
+		g_string_append(gstr, "$Label3 ");
+	if (flags & CAMEL_IMAP_MESSAGE_LABEL4)
+		g_string_append(gstr, "$Label4 ");
+	if (flags & CAMEL_IMAP_MESSAGE_LABEL5)
+		g_string_append(gstr, "$Label5 ");
 
 	if (gstr->str[gstr->len - 1] == ' ')
 		gstr->str[gstr->len - 1] = ')';
@@ -489,6 +502,31 @@ imap_create_flag_list (guint32 flags)
 	return flag_list;
 }
 
+guint32
+imap_label_to_flags(CamelMessageInfo *info)
+{
+	const char *label;
+	guint32 flags;
+
+	label = camel_message_info_user_tag(info, "label");
+	if (label == NULL)
+		flags = 0;
+	else if (!strcmp(label, "important"))
+		flags = CAMEL_IMAP_MESSAGE_LABEL1;
+	else if (!strcmp(label, "work"))
+		flags =  CAMEL_IMAP_MESSAGE_LABEL2;
+	else if (!strcmp(label, "personal"))
+		flags = CAMEL_IMAP_MESSAGE_LABEL3;
+	else if (!strcmp(label, "todo"))
+		flags = CAMEL_IMAP_MESSAGE_LABEL4;
+	else if (!strcmp(label, "later"))
+		flags = CAMEL_IMAP_MESSAGE_LABEL5;
+	else
+		flags = 0;
+
+	return flags;
+}
+
 guint32
 imap_parse_flag_list (char **flag_list_p)
 {
@@ -516,7 +554,19 @@ imap_parse_flag_list (char **flag_list_p)
 		else if (!g_ascii_strncasecmp (flag_list, "\\Recent", len))
 			flags |= CAMEL_IMAP_MESSAGE_RECENT;
 		else if (!g_ascii_strncasecmp(flag_list, "\\*", len))
-			flags |= CAMEL_MESSAGE_USER;
+			flags |= CAMEL_MESSAGE_USER|CAMEL_MESSAGE_JUNK|CAMEL_IMAP_MESSAGE_LABEL_MASK;
+		else if (!g_ascii_strncasecmp(flag_list, "Junk", len))
+			flags |= CAMEL_MESSAGE_JUNK;
+		else if (!g_ascii_strncasecmp(flag_list, "$Label1", len))
+			flags |= CAMEL_IMAP_MESSAGE_LABEL1;
+		else if (!g_ascii_strncasecmp(flag_list, "$Label2", len))
+			flags |= CAMEL_IMAP_MESSAGE_LABEL2;
+		else if (!g_ascii_strncasecmp(flag_list, "$Label3", len))
+			flags |= CAMEL_IMAP_MESSAGE_LABEL3;
+		else if (!g_ascii_strncasecmp(flag_list, "$Label4", len))
+			flags |= CAMEL_IMAP_MESSAGE_LABEL4;
+		else if (!g_ascii_strncasecmp(flag_list, "$Label5", len))
+			flags |= CAMEL_IMAP_MESSAGE_LABEL5;
 		
 		flag_list += len;
 		if (*flag_list == ' ')
@@ -533,7 +583,7 @@ imap_parse_flag_list (char **flag_list_p)
 }
 
 /*
- From rfc2060
+ From RFC3501, which adds resp_specials over RFC2060
 
 ATOM_CHAR       ::= <any CHAR except atom_specials>
 
@@ -556,12 +606,13 @@ resp_specials   ::= "]"
 */
 
 static unsigned char imap_atom_specials[256] = {
+/*	0  1  2  3  4  5  6  7  8  9  A  B  C  D  E  F */
 /* 00 */0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 /* 10 */0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 /* 20 */0, 1, 0, 1, 1, 0, 1, 1, 0, 0, 0, 1, 1, 1, 1, 1,
 /* 30 */1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 /* 40 */1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
-/* 50 */1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 1, 1,
+/* 50 */1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 1, 1, 1,
 /* 60 */1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 /* 70 */1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0,
 	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
@@ -1274,83 +1325,32 @@ imap_mailbox_decode (const unsigned char *in, size_t inlen)
 char *
 imap_path_to_physical (const char *prefix, const char *vpath)
 {
-	const char *p, *newp;
-	char *dp;
-	char *ppath;
-	int ppath_len;
-	int prefix_len;
+	GString *out = g_string_new(prefix);
+	const char *p = vpath;
+	char c, *res;
 
-	while (*vpath == '/')
-		vpath++;
-	if (!prefix)
-		prefix = "";
-
-	/* Calculate the length of the real path. */
-	ppath_len = strlen (vpath);
-	ppath_len++;	/* For the ending zero.  */
-
-	prefix_len = strlen (prefix);
-	ppath_len += prefix_len;
-	ppath_len++;	/* For the separating slash.  */
-
-	/* Take account of the fact that we need to translate every
-	 * separator into `subfolders/'.
-	 */
+	g_string_append_c(out, '/');
 	p = vpath;
-	while (1) {
-		newp = strchr (p, '/');
-		if (newp == NULL)
-			break;
-
-		ppath_len += SUBFOLDER_DIR_NAME_LEN;
-		ppath_len++; /* For the separating slash.  */
-
-		/* Skip consecutive slashes.  */
-		while (*newp == '/')
-			newp++;
-
-		p = newp;
-	};
-
-	ppath = g_malloc (ppath_len);
-	dp = ppath;
-
-	memcpy (dp, prefix, prefix_len);
-	dp += prefix_len;
-	*(dp++) = '/';
-
-	/* Copy the mangled path.  */
-	p = vpath;
- 	while (1) {
-		newp = strchr (p, '/');
-		if (newp == NULL) {
-			strcpy (dp, p);
-			break;
-		}
-
-		memcpy (dp, p, newp - p + 1); /* `+ 1' to copy the slash too.  */
-		dp += newp - p + 1;
-
-		memcpy (dp, SUBFOLDER_DIR_NAME, SUBFOLDER_DIR_NAME_LEN);
-		dp += SUBFOLDER_DIR_NAME_LEN;
-
-		*(dp++) = '/';
-
-		/* Skip consecutive slashes.  */
-		while (*newp == '/')
-			newp++;
-
-		p = newp;
+	while ((c = *p++)) {
+		if (c == '/') {
+			g_string_append(out, "/" SUBFOLDER_DIR_NAME "/");
+			while (*p == '/')
+				p++;
+		} else
+			g_string_append_c(out, c);
 	}
 
-	return ppath;
+	res = out->str;
+	g_string_free(out, FALSE);
+
+	return res;
 }
 
 static gboolean
 find_folders_recursive (const char *physical_path, const char *path,
 			IMAPPathFindFoldersCallback callback, gpointer data)
 {
-	DIR *dir;
+	GDir *dir;
 	char *subfolder_directory_path;
 	gboolean ok;
 
@@ -1368,7 +1368,7 @@ find_folders_recursive (const char *physical_path, const char *path,
 	}
 
 	/* Now scan the subfolders and load them. */
-	dir = opendir (subfolder_directory_path);
+	dir = g_dir_open (subfolder_directory_path, 0, NULL);
 	if (dir == NULL) {
 		g_free (subfolder_directory_path);
 		return TRUE;
@@ -1377,27 +1377,24 @@ find_folders_recursive (const char *physical_path, const char *path,
 	ok = TRUE;
 	while (ok) {
 		struct stat file_stat;
-		struct dirent *dirent;
+		const char *dirent;
 		char *file_path;
 		char *new_path;
 
-		dirent = readdir (dir);
+		dirent = g_dir_read_name (dir);
 		if (dirent == NULL)
 			break;
 
-		if (strcmp (dirent->d_name, ".") == 0 || strcmp (dirent->d_name, "..") == 0)
-			continue;
-
 		file_path = g_strdup_printf ("%s/%s", subfolder_directory_path,
-					     dirent->d_name);
+					     dirent);
 
-		if (stat (file_path, &file_stat) < 0 ||
+		if (g_stat (file_path, &file_stat) < 0 ||
 		    ! S_ISDIR (file_stat.st_mode)) {
 			g_free (file_path);
 			continue;
 		}
 
-		new_path = g_strdup_printf ("%s/%s", path, dirent->d_name);
+		new_path = g_strdup_printf ("%s/%s", path, dirent);
 
 		ok = find_folders_recursive (file_path, new_path, callback, data);
 
@@ -1405,7 +1402,7 @@ find_folders_recursive (const char *physical_path, const char *path,
 		g_free (new_path);
 	}
 
-	closedir (dir);
+	g_dir_close (dir);
 	g_free (subfolder_directory_path);
 
 	return ok;
