@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  Camel
- *  Copyright (C) 1999-2004 Jeffrey Stedfast
+ *  Copyright (C) 1999-2006 Jeffrey Stedfast
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -638,7 +638,7 @@ decode_internaldate (const char *in)
 		return (time_t) -1;
 	
 	for (n = 0; n < 12; n++) {
-		if (!strncasecmp (inptr, tm_months[n], 3))
+		if (!g_ascii_strncasecmp (inptr, tm_months[n], 3))
 			break;
 	}
 	
@@ -764,6 +764,7 @@ imap4_fetch_all_add (struct imap4_fetch_all_t *fetch)
 		
 		if ((info = camel_folder_summary_uid (fetch->summary, camel_message_info_uid (envelope->info)))) {
 			camel_message_info_free (envelope->info);
+			camel_message_info_free (info);
 			g_free (envelope);
 			continue;
 		}
