@@ -431,16 +431,11 @@ connect_to_server_wrapper (CamelService *service, CamelException *ex)
 		camel_exception_clear (ex);
 		ai = camel_getaddrinfo(service->url->host, port, &hints, ex);
 	}
+	
 	if (ai == NULL)
 		return FALSE;
 	
-	if (!(ret = connect_to_server (service, ai, mode, ex)) && mode == MODE_SSL) {
-		camel_exception_clear (ex);
-		ret = connect_to_server (service, ai, MODE_TLS, ex);
-	} else if (!ret && mode == MODE_TLS) {
-		camel_exception_clear (ex);
-		ret = connect_to_server (service, ai, MODE_CLEAR, ex);
-	}
+	ret = connect_to_server (service, ai, mode, ex);
 	
 	camel_freeaddrinfo (ai);
 	
@@ -875,9 +870,10 @@ smtp_set_exception (CamelSmtpTransport *transport, gboolean disconnect, const ch
 static gboolean
 smtp_helo (CamelSmtpTransport *transport, CamelException *ex)
 {
-	/* say hello to the server */
 	char *name = NULL, *cmdbuf = NULL, *respbuf = NULL;
 	const char *token, *numeric = NULL;
+	struct sockaddr *addr;
+	socklen_t addrlen;
 	
 	/* these are flags that we set, so unset them in case we
 	   are being called a second time (ie, after a STARTTLS) */
@@ -892,20 +888,23 @@ smtp_helo (CamelSmtpTransport *transport, CamelException *ex)
 	}
 	
 	camel_operation_start_transient (NULL, _("SMTP Greeting"));
-
+	
+	addr = transport->localaddr;
+	addrlen = transport->localaddrlen;
+	
 	/* force name resolution first, fallback to numerical, we need to know when it falls back */
-	if (camel_getnameinfo(transport->localaddr, transport->localaddrlen, &name, NULL, NI_NAMEREQD, NULL) != 0) {
-		if (camel_getnameinfo(transport->localaddr, transport->localaddrlen, &name, NULL, NI_NUMERICHOST, NULL) != 0)
-			name = g_strdup("localhost.localdomain");
-		else {
-			if (transport->localaddr->sa_family == AF_INET6)
+	if (camel_getnameinfo (addr, addrlen, &name, NULL, NI_NAMEREQD, NULL) != 0) {
+		g_free (name);
+		if (camel_getnameinfo (addr, addrlen, &name, NULL, NI_NUMERICHOST, NULL) != 0) {
+			name = g_strdup ("localhost.localdomain");
+		} else {
+			if (addr->sa_family == AF_INET6)
 				numeric = "IPv6:";
 			else
 				numeric = "";
 		}
 	}
 	
-	/* hiya server! how are you today? */
 	token = (transport->flags & CAMEL_SMTP_TRANSPORT_IS_ESMTP) ? "EHLO" : "HELO";
 	if (numeric)
 		cmdbuf = g_strdup_printf("%s [%s%s]\r\n", token, numeric, name);
