@@ -216,6 +216,10 @@ e_source_group_new_from_xmldoc (xmlDocPtr doc)
 		goto done;
 
 	new = g_object_new (e_source_group_get_type (), NULL);
+
+	if (!new)
+		goto done;
+
 	new->priv->uid = g_strdup (uid);
 
 	e_source_group_set_name (new, name);
@@ -413,7 +417,11 @@ e_source_group_uid_from_xmldoc (xmlDocPtr doc)
 	xmlChar *name;
 	char *retval;
 
-	if (strcmp (root->name, "group") != 0)
+	if (root && root->name) {
+		if (strcmp (root->name, "group") != 0)
+			return NULL;
+	}
+	else 
 		return NULL;
 
 	name = xmlGetProp (root, "uid");
