@@ -1093,7 +1093,7 @@ user_insert_text (ENameSelectorEntry *name_selector_entry, gchar *new_text,
 		chars_inserted++;
 	}
 
-	if (chars_inserted == 1) {
+	if (chars_inserted >= 1) {
 		/* If the user inserted one character, kick off completion */
 		if (!name_selector_entry->update_completions_cb_id) {
 			name_selector_entry->update_completions_cb_id =
@@ -1106,10 +1106,6 @@ user_insert_text (ENameSelectorEntry *name_selector_entry, gchar *new_text,
 				g_idle_add ((GSourceFunc) type_ahead_complete_on_idle_cb,
 					    name_selector_entry);
 		}
-	} else if (chars_inserted > 1 && name_selector_entry->type_ahead_complete_cb_id) {
-		/* If the user inserted more than one character, prevent completion */
-		g_source_remove (name_selector_entry->type_ahead_complete_cb_id);
-		name_selector_entry->type_ahead_complete_cb_id = 0;
 	}
 
 	g_signal_handlers_unblock_by_func (name_selector_entry, user_delete_text, name_selector_entry);
@@ -1485,8 +1481,10 @@ destination_row_changed (ENameSelectorEntry *name_selector_entry, GtkTreePath *p
 	n = gtk_tree_path_get_indices (path) [0];
 	destination = e_destination_store_get_destination (name_selector_entry->destination_store, iter);
 
+	if (!destination)
+		return;
+
 	g_assert (n >= 0);
-	g_assert (destination != NULL);
 
 	entry_text = gtk_entry_get_text (GTK_ENTRY (name_selector_entry));
 	if (!get_range_by_index (entry_text, n, &range_start, &range_end)) {
@@ -1761,6 +1759,7 @@ editor_closed_cb (GtkObject *editor, gpointer data)
 	g_free (contact_uid);
 	g_object_unref (contact);
 	g_object_unref (editor);
+	g_object_unref (name_selector_entry);
 }
 
 static void
@@ -1832,6 +1831,7 @@ popup_activate_contact (ENameSelectorEntry *name_selector_entry, GtkWidget *menu
 			return;
 
 		contact_list_editor = (*name_selector_entry->contact_list_editor_func) (book, contact, FALSE, TRUE);
+		g_object_ref (name_selector_entry);
 		g_signal_connect (contact_list_editor, "editor_closed",
 				  G_CALLBACK (editor_closed_cb), name_selector_entry);
 	} else {
@@ -1841,6 +1841,7 @@ popup_activate_contact (ENameSelectorEntry *name_selector_entry, GtkWidget *menu
 			return;
 
 		contact_editor = (*name_selector_entry->contact_editor_func) (book, contact, FALSE, TRUE);
+		g_object_ref (name_selector_entry);
 		g_signal_connect (contact_editor, "editor_closed",
 				  G_CALLBACK (editor_closed_cb), name_selector_entry);
 	}
